package com.aizuda.boot.modules.flw.service;

import com.aizuda.boot.modules.flw.entity.dto.ExecuteTaskDTO;
import com.aizuda.boot.modules.flw.entity.dto.ProcessTaskDTO;
import com.aizuda.boot.modules.flw.entity.dto.RejectTaskDTO;
import com.aizuda.boot.modules.flw.entity.vo.PendingApprovalTaskVO;
import com.aizuda.boot.modules.flw.entity.vo.ProcessTaskVO;
import com.aizuda.core.api.PageParam;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flowlong.bpm.engine.core.FlowCreator;
import com.flowlong.bpm.engine.entity.FlwTaskActor;

/**
 * 流程任务 服务类
 *
 * @author 青苗
 * @since 2023-12-11
 */
public interface IProcessTaskService {

    /**
     * 待认领任务分页列表
     */
    Page<ProcessTaskVO> pagePendingClaim(PageParam<ProcessTaskDTO> pageParam);

    /**
     * 待审批任务分页列表
     */
    Page<PendingApprovalTaskVO> pagePendingApproval(PageParam<ProcessTaskDTO> pageParam);

    /**
     * 我的申请任务分页列表
     */
    Page<ProcessTaskVO> pageMyApplication(PageParam<ProcessTaskDTO> pageParam);

    /**
     * 我收到的任务分页列表
     */
    Page<ProcessTaskVO> pageMyReceived(PageParam<ProcessTaskDTO> pageParam);

    /**
     * 已审批任务分页列表
     */
    Page<ProcessTaskVO> pageApproved(PageParam<ProcessTaskDTO> pageParam);

    /**
     * 设置任务为已阅状态
     *
     * @param taskId      任务ID
     * @param flowCreator 流程创建者
     */
    boolean viewed(Long taskId, FlowCreator flowCreator);

    /**
     * 转办任务
     *
     * @param taskId            任务ID
     * @param taskActor         任务参与者
     * @param assigneeTaskActor 任务办理人
     */
    boolean transfer(Long taskId, FlwTaskActor taskActor, FlwTaskActor assigneeTaskActor);

    /**
     * 委派任务
     *
     * @param taskId            任务ID
     * @param taskActor         任务参与者
     * @param assigneeTaskActor 任务办理人
     */
    boolean delegate(Long taskId, FlwTaskActor taskActor, FlwTaskActor assigneeTaskActor);

    /**
     * 拿回任务
     *
     * @param taskId      任务ID
     * @param flowCreator 任务创建者
     */
    boolean reclaim(Long taskId, FlowCreator flowCreator);

    /**
     * 认领任务
     *
     * @param taskId      任务ID
     * @param flowCreator 任务创建者
     */
    boolean resume(Long taskId, FlowCreator flowCreator);

    /**
     * 执行节点跳转任务
     *
     * @param taskId      任务ID
     * @param nodeName    节点名称
     * @param flowCreator 任务创建者
     */
    boolean executeJump(Long taskId, String nodeName, FlowCreator flowCreator);

    /**
     * 执行流程任务
     *
     * @param flowCreator 流程创建者
     * @param dto         执行任务对象 {@link ExecuteTaskDTO}
     */
    boolean execute(FlowCreator flowCreator, ExecuteTaskDTO dto);

    /**
     * 驳回至上一步任务
     *
     * @param flowCreator 流程创建者
     * @param dto         执行任务对象 {@link RejectTaskDTO}
     */
    boolean reject(FlowCreator flowCreator, RejectTaskDTO dto);
}
