package com.aizuda.boot.modules.flw.service.impl;

import com.aizuda.boot.modules.flw.entity.dto.ExecuteTaskDTO;
import com.aizuda.boot.modules.flw.entity.dto.ProcessTaskDTO;
import com.aizuda.boot.modules.flw.entity.dto.RejectTaskDTO;
import com.aizuda.boot.modules.flw.entity.vo.PendingApprovalTaskVO;
import com.aizuda.boot.modules.flw.entity.vo.ProcessTaskVO;
import com.aizuda.boot.modules.flw.mapper.FlowlongMapper;
import com.aizuda.boot.modules.flw.service.IProcessTaskService;
import com.aizuda.core.api.ApiAssert;
import com.aizuda.core.api.PageParam;
import com.aizuda.service.web.UserSession;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flowlong.bpm.engine.FlowLongEngine;
import com.flowlong.bpm.engine.TaskService;
import com.flowlong.bpm.engine.core.FlowCreator;
import com.flowlong.bpm.engine.entity.FlwTask;
import com.flowlong.bpm.engine.entity.FlwTaskActor;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

/**
 * 流程任务 服务实现类
 *
 * @author 青苗
 * @since 2023-12-11
 */
@Service
public class ProcessTaskServiceImpl implements IProcessTaskService {
    @Resource
    private FlowlongMapper flowlongMapper;
    @Resource
    private FlowLongEngine flowLongEngine;

    @Override
    public Page<ProcessTaskVO> pagePendingClaim(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPagePendingClaim(pageParam.page(), dto);
    }

    @Override
    public Page<PendingApprovalTaskVO> pagePendingApproval(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPagePendingApproval(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageMyApplication(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageMyApplication(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageMyReceived(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageMyReceived(pageParam.page(), dto);
    }

    @Override
    public Page<ProcessTaskVO> pageApproved(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = this.getProcessTaskDTO(pageParam);
        return flowlongMapper.selectPageApproved(pageParam.page(), dto);
    }

    @Override
    public boolean viewed(Long taskId, FlowCreator flowCreator) {
        return flowLongEngine.taskService().viewTask(taskId, FlwTaskActor.ofFlowCreator(flowCreator));
    }

    /**
     * 获取流程任务DTO
     */
    private ProcessTaskDTO getProcessTaskDTO(PageParam<ProcessTaskDTO> pageParam) {
        ProcessTaskDTO dto = pageParam.getData();
        if (null == dto) {
            dto = new ProcessTaskDTO();
        }
        UserSession userSession = UserSession.getLoginInfo();
        dto.setCreateId(userSession.getUserId());
        return dto;
    }

    @Override
    public boolean transfer(Long taskId, FlwTaskActor taskActor, FlwTaskActor assigneeTaskActor) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.transferTask(taskId, taskActor, assigneeTaskActor);
    }

    @Override
    public boolean delegate(Long taskId, FlwTaskActor taskActor, FlwTaskActor assigneeTaskActor) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.delegateTask(taskId, taskActor, assigneeTaskActor);
    }

    @Override
    public boolean reclaim(Long taskId, FlowCreator flowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return taskService.reclaimTask(taskId, flowCreator).isPresent();
    }

    @Override
    public boolean resume(Long taskId, FlowCreator flowCreator) {
        TaskService taskService = flowLongEngine.taskService();
        return null != taskService.resume(taskId, FlwTaskActor.ofFlowCreator(flowCreator));
    }

    @Override
    public boolean executeJump(Long taskId, String nodeName, FlowCreator flowCreator) {
        return flowLongEngine.executeJumpTask(taskId, nodeName, flowCreator);
    }

    @Override
    public boolean execute(FlowCreator flowCreator, ExecuteTaskDTO dto) {
        return flowLongEngine.executeTask(dto.getTaskId(), flowCreator, dto.getArgs());
    }

    @Override
    public boolean reject(FlowCreator flowCreator, RejectTaskDTO dto) {
        FlwTask flwTask = flowLongEngine.queryService().getTask(dto.getTaskId());
        ApiAssert.isEmpty(flwTask, "当前ID执行任务不存在");
        TaskService taskService = flowLongEngine.taskService();
        return taskService.rejectTask(flwTask, flowCreator, dto.getArgs()).isPresent();
    }
}
